@@ -5,7 +5,7 @@
 
 package XML::Compile::Schema;
 use vars '$VERSION';
-$VERSION = '1.18';
+$VERSION = '1.19';
 
 use base 'XML::Compile';
 
@@ -480,4 +480,10 @@ sub printIndex(@)
 }
 
 
+sub doesExtend($$)
+{   my $self = shift;
+    $self->namespaces->doesExtend(@_);
+}
+
+
 1;
