@@ -7,7 +7,7 @@ use strict;
 
 package XML::Compile::Schema::BuiltInTypes;
 use vars '$VERSION';
-$VERSION = '1.18';
+$VERSION = '1.19';
 
 use base 'Exporter';
 
@@ -48,11 +48,24 @@ sub _collapse { local $_ = $_[0]; s/[\t\r\n]+/ /g; s/^ +//; s/ +$//; $_}
 
 # format not useful, because xsi:type not supported
 $builtin_types{anySimpleType} =
+ { example => 'anySimple'
+ , parse   => sub {shift}
+ , extends => 'anyType'
+ };
+
 $builtin_types{anyType} =
  { example => 'anything'
  , parse   => sub {shift}
+ , extends => undef         # the root type
  };
 
+$builtin_types{anyAtomicType} =
+ { example => 'anyAtomic'
+ , parse   => sub {shift}
+ , extends => 'anySimpleType'
+ };
+
+
 $builtin_types{error}   = {example => '[some error structure]'};
 
 
@@ -62,6 +75,7 @@ $builtin_types{boolean} =
                   : $_[0] ? 1 : 0 }
  , check   => sub { $_[0] =~ m/^\s*(?:false|true|0|1)\s*$/i }
  , example => 'true'
+ , extends => 'anyAtomicType'
  };
 
 
@@ -81,6 +95,7 @@ $builtin_types{integer} =
  { parse   => \&bigint
  , check   => sub { $_[0] =~ m/^\s*[-+]?\s*\d[\s\d]*$/ }
  , example => 42
+ , extends => 'decimal'
  };
 
 
@@ -88,6 +103,7 @@ $builtin_types{negativeInteger} =
  { parse   => \&bigint
  , check   => sub { $_[0] =~ m/^\s*\-\s*\d[\s\d]*$/ }
  , example => '-1'
+ , extends => 'nonPositiveInteger'
  };
 
 
@@ -95,6 +111,7 @@ $builtin_types{nonNegativeInteger} =
  { parse   => \&bigint
  , check   => sub { $_[0] =~ m/^\s*(?:\+\s*)?\d[\s\d]*$/ }
  , example => '17'
+ , extends => 'integer'
  };
 
 
@@ -102,6 +119,7 @@ $builtin_types{positiveInteger} =
  { parse   => \&bigint
  , check   => sub { $_[0] =~ m/^\s*(?:\+\s*)?\d[\s\d]*$/ && $_[0] =~ m/[1-9]/ }
  , example => '+3'
+ , extends => 'nonNegativeInteger'
  };
 
 
@@ -110,6 +128,7 @@ $builtin_types{nonPositiveInteger} =
  , check   => sub { $_[0] =~ m/^\s*(?:\-\s*)?\d[\s\d]*$/
                  || $_[0] =~ m/^\s*(?:\+\s*)0[0\s]*$/ }
  , example => '-42'
+ , extends => 'integer'
  };
 
 
@@ -118,6 +137,7 @@ $builtin_types{long} =
  , check   =>
      sub { $_[0] =~ m/^\s*[-+]?\s*\d[\s\d]*$/ && ($_[0] =~ tr/0-9//) < 20 }
  , example => '-100'
+ , extends => 'integer'
  };
 
 
@@ -125,13 +145,15 @@ $builtin_types{unsignedLong} =
  { parse   => \&bigint
  , check   => sub {$_[0] =~ m/^\s*\+?\s*\d[\s\d]*$/ && ($_[0] =~ tr/0-9//) < 21}
  , example => '100'
+ , extends => 'nonNegativeInteger'
  };
 
 
 $builtin_types{unsignedInt} =
  { parse   => \&bigint
- , check   => sub {$_[0] =~ m/^\s*\+?\s*\d[\s\d]*$/ && ($_[0] =~ tr/0-9//) <10}
+ , check   => sub {$_[0] =~ m/^\s*\+?\s*\d[\s\d]*$/ && ($_[0] =~ tr/0-9//) <=10}
  , example => '42'
+ , extends => 'unsignedLong'
  };
 
 # Used when 'sloppy_integers' was set: the size of the values
@@ -171,6 +193,7 @@ $builtin_types{int} =
  , format  => \&int2str
  , check   => sub {$_[0] =~ m/^\s*[+-]?\d+\s*$/}
  , example => '42'
+ , extends => 'long'
  };
 
 
@@ -180,6 +203,7 @@ $builtin_types{short} =
  , check   =>
     sub { $_[0] =~ m/^\s*[+-]?\d+\s*$/ && $_[0] >= -32768 && $_[0] <= 32767 }
  , example => '-7'
+ , extends => 'int'
  };
 
 
@@ -189,6 +213,7 @@ $builtin_types{unsignedShort} =
  , check  =>
     sub { $_[0] =~ m/^\s*[+-]?\d+\s*$/ && $_[0] >= 0 && $_[0] <= 65535 }
  , example => '7'
+ , extends => 'unsignedInt'
  };
 
 
@@ -197,6 +222,7 @@ $builtin_types{byte} =
  , format  => \&int2str
  , check   => sub {$_[0] =~ m/^\s*[+-]?\d+\s*$/ && $_[0] >= -128 && $_[0] <=127}
  , example => '-2'
+ , extends => 'short'
  };
 
 
@@ -205,13 +231,15 @@ $builtin_types{unsignedByte} =
  , format  => \&int2str
  , check   => sub {$_[0] =~ m/^\s*[+-]?\d+\s*$/ && $_[0] >= 0 && $_[0] <= 255}
  , example => '2'
+ , extends => 'unsignedShort'
  };
 
 
 $builtin_types{decimal} =
  { parse   => sub {$_[0] =~ s/\s+//g; Math::BigFloat->new($_[0]) }
- , check   => sub {$_[0] =~ m#^(\+|\-)?([0-9]+(\.[0-9]*)?|\.[0-9]+)$#}
+ , check   => sub {$_[0] =~ m/^(\+|\-)?([0-9]+(\.[0-9]*)?|\.[0-9]+)$/}
  , example => '3.1415'
+ , extends => 'anyAtomicType'
  };
 
 
@@ -247,6 +275,7 @@ $builtin_types{double} =
  , format  => \&num2str
  , check   => \&numcheck
  , example => '3.1415'
+ , extends => 'anyAtomicType'
  };
 
 $builtin_types{sloppy_float} =
@@ -255,6 +284,7 @@ $builtin_types{sloppy_float} =
       $@ ? undef : 1;
     }
  , example => '3.1415'
+ , extends => 'anyAtomicType'
  };
 
 
@@ -263,6 +293,7 @@ $builtin_types{base64Binary} =
  , format  => sub { eval { encode_base64 $_[0] } }
  , check   => sub { !$@ }
  , example => 'VGVzdA=='
+ , extends => 'anyAtomicType'
  };
 
 
@@ -273,6 +304,7 @@ $builtin_types{hexBinary} =
  , check   =>
      sub { $_[0] !~ m/[^0-9a-fA-F\s]/ && (($_[0] =~ tr/0-9a-fA-F//) %2)==0}
  , example => 'F00F'
+ , extends => 'anyAtomicType'
  };
 
 
@@ -295,6 +327,7 @@ $builtin_types{date} =
  , format  => sub { $_[0] =~ /\D/ ? $_[0] : strftime("%Y-%m-%d", gmtime $_[0])}
  , check   => sub { (my $val = $_[0]) =~ s/\s+//g; $val =~ $date }
  , example => '2006-10-06'
+ , extends => 'anyAtomicType'
  };
 
 
@@ -307,19 +340,36 @@ $builtin_types{time} =
       strftime "%T$subsec", gmtime $_[0] }
  , check   => sub { (my $val = $_[0]) =~ s/\s+//g; $val =~ $time }
  , example => '11:12:13'
+ , extends => 'anyAtomicType'
  };
 
 
-my $dateTime = qr/^ $yearFrag \- $monthFrag \- $dayFrag
-                    T $timeFrag $timezoneFrag? $/x;
+my $dateTime
+  = qr/^ $yearFrag \- $monthFrag \- $dayFrag T $timeFrag $timezoneFrag? $/x;
+my $dateTimeStamp
+  = qr/^ $yearFrag \- $monthFrag \- $dayFrag T $timeFrag $timezoneFrag $/x;
+
+sub _dt_format
+{   return $_[0] if $_[0] =~ /[^0-9.]/;  # already formated
+    my $subsec = $_[0] =~ /(\.\d+)/ ? $1 : '';
+    strftime "%Y-%m-%dT%H:%M:%S${subsec}Z", gmtime $_[0];
+}
 
 $builtin_types{dateTime} =
  { parse   => \&_collapse
- , format  => sub { return $_[0] if $_[0] =~ /[^0-9.]/;
-       my $subsec = $_[0] =~ /(\.\d+)/ ? $1 : '';
-       strftime "%Y-%m-%dT%H:%M:%S${subsec}Z", gmtime $_[0] }
+ , format  => \&_dt_format
  , check   => sub { (my $val = $_[0]) =~ s/\s+//g; $val =~ $dateTime }
  , example => '2006-10-06T00:23:02Z'
+ , extends => 'anyAtomicType'
+ };
+
+
+$builtin_types{dateTimeStamp} =
+ { parse   => \&_collapse
+ , format  => \&_dt_format
+ , check   => sub { (my $val = $_[0]) =~ s/\s+//g; $val =~ $dateTimeStamp }
+ , example => '2006-10-06T00:23:02Z'
+ , extends => 'dateTime'
  };
 
 
@@ -328,6 +378,7 @@ $builtin_types{gDay} =
  { parse   => \&_collapse
  , check   => sub { (my $val = $_[0]) =~ s/\s+//g; $val =~ $gDay }
  , example => '---12+09:00'
+ , extends => 'anyAtomicType'
  };
 
 
@@ -336,6 +387,7 @@ $builtin_types{gMonth} =
  { parse   => \&_collapse
  , check   => sub { (my $val = $_[0]) =~ s/\s+//g; $val =~ $gMonth }
  , example => '--09+07:00'
+ , extends => 'anyAtomicType'
  };
 
 
@@ -344,6 +396,7 @@ $builtin_types{gMonthDay} =
  { parse   => \&_collapse
  , check   => sub { (my $val = $_[0]) =~ s/\s+//g; $val =~ $gMonthDay }
  , example => '--09-12+07:00'
+ , extends => 'anyAtomicType'
  };
 
 
@@ -352,6 +405,7 @@ $builtin_types{gYear} =
  { parse   => \&_collapse
  , check   => sub { (my $val = $_[0]) =~ s/\s+//g; $val =~ $gYear }
  , example => '2006+07:00'
+ , extends => 'anyAtomicType'
  };
 
 
@@ -360,6 +414,7 @@ $builtin_types{gYearMonth} =
  { parse   => \&_collapse
  , check   => sub { (my $val = $_[0]) =~ s/\s+//g; $val =~ $gYearMonth }
  , example => '2006-11+07:00'
+ , extends => 'anyAtomicType'
  };
 
 
@@ -378,6 +433,7 @@ $builtin_types{dayTimeDuration} =
  , check  => sub { my $val = $_[0]; $val =~ s/\s+//g; $val =~
      m/^\-?P(?:\d+D)?(?:T(?:\d+H)?(?:\d+M)?(?:\d+(?:\.\d+)?S)?)?$/ }
  , example => 'P2DT3H5M10S'
+ , extends => 'duration'
  };
 
 
@@ -386,17 +442,20 @@ $builtin_types{yearMonthDuration} =
  , check  => sub { my $val = $_[0]; $val =~ s/\s+//g; $val =~
      m/^\-?P(?:\d+Y)?(?:\d+M)?$/ }
  , example => 'P40Y5M'
+ , extends => 'duration'
  };
 
 
 $builtin_types{string} =
  { example => 'example'
+ , extends => 'anyAtomicType'
  };
 
 
 $builtin_types{normalizedString} =
  { parse   => \&_replace
  , example => 'example'
+ , extends => 'string'
  };
 
 
@@ -405,6 +464,7 @@ $builtin_types{language} =
  , check   => sub { my $v = $_[0]; $v =~ s/\s+//g; $v =~
        m/^[a-zA-Z]{1,8}(?:\-[a-zA-Z0-9]{1,8})*$/ }
  , example => 'nl-NL'
+ , extends => 'token'
  };
 
 
@@ -422,20 +482,29 @@ $builtin_types{ID} =
  { parse   => \&_collapse
  , check   => \&_ncname
  , example => 'id_'.$ids++
+ , extends => 'NCName'
  };
 
 $builtin_types{IDREF} =
  { parse   => \&_collapse
  , check   => \&_ncname
  , example => 'id-ref'
+ , extends => 'NCName'
  };
 
 
 $builtin_types{NCName} =
-$builtin_types{ENTITY} =
  { parse   => \&_collapse
  , check   => \&_ncname
  , example => 'label'
+ , extends => 'Name'
+ };
+
+$builtin_types{ENTITY} =
+ { parse   => \&_collapse
+ , check   => \&_ncname
+ , example => 'entity'
+ , extends => 'NCName'
  };
 
 $builtin_types{IDREFS} =
@@ -445,24 +514,28 @@ $builtin_types{ENTITIES} =
  , check   => sub { $_[0] !~ m/\:/ }
  , example => 'labels'
  , is_list => 1
+ , extends => 'anySimpleType'
  };
 
 
 $builtin_types{Name} =
  { parse   => \&_collapse
  , example => 'name'
+ , extends => 'token'
  };
 
 
 $builtin_types{token} =
  { parse   => \&_collapse
  , example => 'token'
+ , extends => 'normalizedString'
  };
 
 # check required!  \c
 $builtin_types{NMTOKEN} =
  { parse   => sub { $_[0] =~ s/\s+//g; $_[0] }
  , example => 'nmtoken'
+ , extends => 'token'
  };
 
 $builtin_types{NMTOKENS} =
@@ -470,6 +543,7 @@ $builtin_types{NMTOKENS} =
  , format  => sub { my $v = shift; ref $v eq 'ARRAY' ? join(' ',@$v) : $v }
  , example => 'nmtokens'
  , is_list => 1
+ , extends => 'anySimpleType'
  };
 
 
@@ -481,6 +555,7 @@ $builtin_types{NMTOKENS} =
 $builtin_types{anyURI} =
   { parse   => \&_collapse
   , example => 'http://example.com'
+  , extends => 'anyAtomicType'
   };
 
 
@@ -508,10 +583,14 @@ $builtin_types{QName} =
           length $def->{prefix} ? "$def->{prefix}:$local" : $local;
         }
  , example => 'myns:local'
+ , extends => 'anyAtomicType'
  };
 
 
-$builtin_types{NOTATION} = {};
+$builtin_types{NOTATION} =
+ {
+   extends => 'anyAtomicType'
+ };
 
 
 $builtin_types{binary} = { example => 'binary string' };
